import logging
from email.mime.application import MIMEApplication
from django.template.loader import render_to_string
import boto3
from botocore.exceptions import ClientError
from django.conf import settings
from django.core.mail import EmailMessage

logger = logging.getLogger('dict_config_logger')


def email_verification(email):
    """Function to send email verification"""
    ses = boto3.client('ses')
    check = check_if_email_verified(email)

    if check:
        logger.info("Email is sent for Verification")

        response = ses.verify_email_identity(
            EmailAddress=email
        )

        logger.info(response)


def check_if_email_verified(email):
    """Function to check if email id from user is verified """
    list_emails = list_email_verified()
    if email in list_emails:
        logger.info("Email is already Verified")
        return False
    return True


def list_email_verified():
    """Function to return list of verified emails """

    ses = boto3.client('ses')
    response = ses.list_identities(
        IdentityType='EmailAddress',
        MaxItems=10
    )
    logger.info(response['Identities'])
    return response['Identities']


def send_notifications(email, sender, email_configuration):
    """This function sends email of a log file """

    logger.info('Sending email to recipients')
    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = sender
    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.

    RECIPIENT = email
    # The subject line for the email.
    SUBJECT = email_configuration.get('Subject')
    LOG_PATH = email_configuration.get('Log_path')

    # The full path to the file that will be attached to the email.
    ATTACHMENT = LOG_PATH
    cover = email_configuration.get('Banner')
    logo = email_configuration.get('Logo')
    paragraph = email_configuration.get('Email_Content')
    # # The HTML body of the email.
    file_name = email_configuration.get('HTML_File')
    # email_html = open(file_name)
    BODY_HTML = open('/HTML_files/' + file_name).read().format(cover=cover,
                                                           logo=logo,
                                                           paragraph=paragraph)
    # Define the attachment part and encode it using MIMEApplication.
    att = MIMEApplication(open(ATTACHMENT, 'rb').read())

    # Add a header to tell the email client to treat this part as an
    # attachment, and to give the attachment a name.
    att.add_header('Content-Disposition', 'attachment',
                   filename="OpenLXP notifications ")

    for each_recipient in RECIPIENT:
        try:
            # Provide the contents of the email.

            mail = EmailMessage(SUBJECT, BODY_HTML, SENDER,
                                [each_recipient])
            mail.content_subtype = "html"
            # Add the attachment to the parent container.
            mail.attach(att)
            mail.send()
            logging.FileHandler(getattr(settings, "LOG_PATH", None),
                                mode='w')
        # Display an error if something goes wrong.
        except ClientError as e:
            logger.error(e.response['Error']['Message'])
            continue


def send_notifications_with_msg(email, sender, msg):
    """This function sends email of a log file """

    logger.info('Sending email to recipients')
    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = sender
    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.
    RECIPIENT = email
    # The subject line for the email.
    SUBJECT = "New Message From OpenLXP Portal"

    # # The HTML body of the email.
    BODY_HTML = """\
       <html>
       <head></head>
       <body>
       <p>""" + msg + """</p>
       </body>
       </html>
       """

    for each_recipient in RECIPIENT:
        try:
            # Provide the contents of the email.

            mail = EmailMessage(SUBJECT, BODY_HTML, SENDER,
                                [each_recipient])
            mail.content_subtype = "html"
            # Add the attachment to the parent container.
            mail.send()
        # Display an error if something goes wrong.
        except ClientError as e:
            logger.error(e.response['Error']['Message'])
            continue



from django.test import TestCase


class TestSetUp(TestCase):
    """Class with setup and teardown for tests in XIS"""

    def setUp(self):
        """Function to set up necessary data for testing"""

        # globally accessible data sets

        self.supplemental_api_endpoint = 'http://openlxp-xis:8020' \
                                         '/api/supplemental-data/'

        self.receive_email_list = ['receiver1@openlxp.com',
                                   'receiver1@openlxp.com']
        self.sender_email = "sender@openlxp.com"

        return super().setUp()

    def tearDown(self):
        return super().tearDown()



import logging

from django.core.management.base import BaseCommand

from openlxp_notifications.management.utils.notification import \
    send_notifications, send_notifications_with_msg
from openlxp_notifications.models import (ReceiverEmailConfiguration,
                                          SenderEmailConfiguration,
                                          EmailConfiguration)

logger = logging.getLogger('dict_config_logger')


def send_log_email():
    """ function to send emails of log file to personas"""

    # getting email id to send email to personas

    email_data = ReceiverEmailConfiguration.objects.values_list(
        'email_address',
        flat=True)
    email = list(email_data)
    # Getting sender email id
    sender_email_configuration = SenderEmailConfiguration.objects.first()
    sender = sender_email_configuration.sender_email_address

    email_configuration = EmailConfiguration.objects.values('Subject',
                                                            'Logo',
                                                            'Banner',
                                                            'Log_path',
                                                            'Email_Content',
                                                            'HTML_File'
                                                            ).first()
    # SUBJECT = email_configuration.SUBJECT
    # HTML_File = email_configuration.HTML_File
    # LOG_PATH = email_configuration.LOG_PATH
    send_notifications(email, sender, email_configuration)


def send_log_email_with_msg(email, sender, msg):
    """ function to send emails of log file to personas"""

    # Getting sender email id
    send_notifications_with_msg(email, sender, msg)


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def handle(self, *args, **options):
        """Email log notification is sent to filer/personas when warning/error
        occurred in EVTVL process"""
        send_log_email()



# Generated by Django 3.1.13 on 2021-10-20 01:24

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('Subject', models.CharField(max_length=200, null=True)),
                ('Logo', models.ImageField(default='logo.jpg', help_text='Please upload your logo here', upload_to='logo_pic')),
                ('Banner', models.ImageField(default='banner.jpg', help_text='Please upload your banner here', upload_to='banner_pic')),
                ('Log_path', models.CharField(max_length=200, null=True)),
                ('Email_Content', models.TextField(max_length=200, null=True)),
                ('HTML_File', models.FileField(help_text='Upload the html file', upload_to='HTML_files')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ReceiverEmailConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('email_address', models.EmailField(help_text='Enter email personas addresses to send log data', max_length=254, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SenderEmailConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('sender_email_address', models.EmailField(help_text='Enter sender email address to send log data from', max_length=254)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging
from unittest.mock import patch

from ddt import ddt
from django.test import tag

from openlxp_notifications.management.utils.notification import (
    check_if_email_verified, send_notifications, send_notifications_with_msg)

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('unit')
@ddt
class UtilsTests(TestSetUp):
    """Unit Test cases for utils """

    # Test cases for NOTIFICATION
    def test_send_notifications(self):
        """Test for function to send emails of log file to personas"""
        with patch('openlxp_notifications.management.utils.notification'
                   '.EmailMessage') as mock_send, \
                patch('openlxp_notifications.management.utils.notification'
                      '.boto3.client'):
            send_notifications(self.receive_email_list, self.sender_email)
            self.assertEqual(mock_send.call_count, 2)

    def test_check_if_email_verified(self):
        """Test to check if email id from user is verified """
        with patch('openlxp_notifications.management.utils.notification'
                   '.list_email_verified') as mock_list:
            mock_list.return_value = self.receive_email_list
            email_value = 'receiver1@openlxp.com'
            return_val = check_if_email_verified(email_value)
            self.assertFalse(return_val)

    def test_check_if_email_not_verified(self):
        """Test to check if email id from user is verified """
        with patch('openlxp_notifications.management.utils.notification'
                   '.list_email_verified') as mock_list:
            mock_list.return_value = self.receive_email_list
            email_value = 'receiver2@openlxp.com'
            return_val = check_if_email_verified(email_value)
            self.assertTrue(return_val)

    def test_send_notifications_with_msg(self):
        """Test for function to send emails of log file to personas"""
        with patch('openlxp_notifications.management.utils.notification'
                   '.EmailMessage') as mock_send, \
                patch('openlxp_notifications.management.utils.notification'
                      '.boto3.client'):
            send_notifications_with_msg(self.receive_email_list,
                                        self.sender_email, 'Message')
            self.assertEqual(mock_send.call_count, 2)



from django.contrib import admin

from .models import ReceiverEmailConfiguration, SenderEmailConfiguration, \
    EmailConfiguration


@admin.register(ReceiverEmailConfiguration)
class ReceiverEmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('email_address',)


@admin.register(SenderEmailConfiguration)
class SenderEmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('sender_email_address',)


@admin.register(EmailConfiguration)
class EmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('Subject',
                    'Logo',
                    'Banner',
                    'Log_path',
                    'Email_Content',
                    'HTML_File',)



from django.test import SimpleTestCase, tag

from openlxp_notifications.models import (ReceiverEmailConfiguration,
                                          SenderEmailConfiguration)


@tag('unit')
class ModelTests(SimpleTestCase):

    def test_create_sender_email_config(self):
        """Test that creating a new Sender Email Configuration entry is
        successful with defaults """
        sender_email_address = 'example@test.com'

        sender_email_Config = SenderEmailConfiguration(
            sender_email_address=sender_email_address)

        self.assertEqual(sender_email_Config.sender_email_address,
                         sender_email_address)

    def test_create_receiver_email_config(self):
        """Test that creating a new Receiver Email Configuration entry is
        successful with defaults """
        email_address = 'example@test.com'

        receiver_email_Config = ReceiverEmailConfiguration(
            email_address=email_address)

        self.assertEqual(receiver_email_Config.email_address,
                         email_address)



import logging
from unittest.mock import patch

from ddt import ddt
from django.test import tag

from openlxp_notifications.management.commands.conformance_alerts import \
    send_log_email
from openlxp_notifications.models import (ReceiverEmailConfiguration,
                                          SenderEmailConfiguration)

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('unit')
@ddt
class CommandTests(TestSetUp):

    # Test cases for conformance_alerts

    def test_send_log_email(self):
        """Test for function to send emails of log file to personas"""
        with patch('openlxp_notifications.management.commands.'
                   'conformance_alerts.ReceiverEmailConfiguration') \
                as receive_email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.SenderEmailConfiguration') \
                as sender_email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.send_notifications',
                    return_value=None
                ) as mock_send_notification:
            receive_email = ReceiverEmailConfiguration(
                email_address=self.receive_email_list)
            receive_email_cfg.first.return_value = receive_email

            send_email = SenderEmailConfiguration(
                sender_email_address=self.sender_email)
            sender_email_cfg.first.return_value = send_email
            send_log_email()
            self.assertEqual(mock_send_notification.call_count, 1)
